﻿using System.Runtime.Serialization;
using System.Text;
using System.Xml;
using System.Xml.Linq;

namespace Infrastructure.Utils.Xml;

public static class DataContractSerializeHelper
{
    private static void RemoveNillableEmptyElements(this XDocument doc)
    {
        var empties = doc.Descendants().Where(x =>
        {
            var nilAttribute = x.Attributes().FirstOrDefault(a => a.Name.LocalName == "nil");
            return nilAttribute is { Value: "true" } && (x.IsEmpty || string.IsNullOrWhiteSpace(x.Value));
        }).ToList();

        empties.ForEach(e => e.Remove());
    }

    public static string Serialize<T>(T obj)
    {
        XDocument doc;

        var type = obj.GetType();

        using (var stream = new MemoryStream())
        {
            using (var writer = XmlDictionaryWriter.CreateTextWriter(stream))
            {
                var serializer = new DataContractSerializer(type);
                serializer.WriteObject(writer, obj);
                writer.Flush();
                stream.Position = 0;
                doc = XDocument.Load(stream);
            }
        }

        // The serializer converts undefined values into empty xml elements. To conform
        // to the <xs:choice> constraints in the xsd we remove the nillable and empty elements after
        // serialization.
        if (!type.Name.Contains("Header"))
        {
            doc.RemoveNillableEmptyElements();
        }

        return doc.ToString();
    }

    public static T Deserialize<T>(string xml) => (T)Deserialize(typeof(T), xml);

    public static object Deserialize(Type type, string xml)
    {
        using Stream stream = new MemoryStream();
        var data = Encoding.UTF8.GetBytes(xml);
        stream.Write(data, 0, data.Length);
        stream.Position = 0;
        var serializer = new DataContractSerializer(type);

        return serializer.ReadObject(stream);
    }
}