﻿using AutoMapper;
using SoapEndpoint.Service.Proxy.Infrastructure;
using SoapEndpoint.Service.Proxy.Models.Systeem;
using SoapEndpoint.Service.Systeem.v2.MessageBoxService;

namespace SoapEndpoint.Service.Proxy.ServiceProxies.Systeem.v2;

public class MessageBoxProxy : IMessageBoxProxy
{
    private readonly IMapper _mapper;
    private readonly ContentResolver _resolver;
    private readonly IMessageBoxService _service;

    public MessageBoxProxy(IMapper mapper, IMessageBoxService service)
    {
        _mapper = mapper;
        _service = service;
        _resolver = new ContentResolver(mapper);
    }

    public async Task<MessageboxConfirmatieResultBerichtModel> ConfirmBatchAsync(
        MessageboxConfirmatieBerichtModel message)
    {
        var request = _mapper.Map<MessageboxConfirmatieBericht>(message);
        var response = await _service.ConfirmBatchAsync(request);
        return _mapper.Map<MessageboxConfirmatieResultBerichtModel>(response);
    }

    public ShvKnooppuntFoutberichtModel GetErrorMessage<TErrorMessage>(string content) where TErrorMessage : class =>
        _resolver.Deserialize<ShvKnooppuntFoutberichtModel, TErrorMessage>(content);

    public async Task<BijlageModel> DownloadAttachmentAsync(DownloadBijlageModel downloadBijlage)
    {
        var request = _mapper.Map<DownloadBijlage>(downloadBijlage);
        var response = await _service.DownloadAttachmentAsync(request);

        return _mapper.Map<BijlageModel>(response);
    }

    public async Task<MessageboxBerichtinformatieModel?> GetMessagesAsync()
    {
        var response = await _service.GetMessagesAsync();
        return _mapper.Map<MessageboxBerichtinformatieModel>(response);
    }
}